#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "RTClib.h"
#include <Servo.h>
#define servoPin 3
#define ECHOPIN 4
#define TRIGPIN 5
#define OPEN 1
#define CLOSE 0
#define heightPerDistance 0.4

RTC_DS1307 rtc;
LiquidCrystal_I2C lcd(0x27, 20, 4);
DateTime now;

//Input & Button Logic
const int numOfInputs = 4;
const int inputPins[numOfInputs] = {8, 9, 10, 11};
int inputState[numOfInputs];
int lastInputState[numOfInputs] = {LOW, LOW, LOW, LOW};
bool inputFlags[numOfInputs] = {LOW, LOW, LOW, LOW};
long lastDebounceTime[numOfInputs] = {0, 0, 0, 0};
long debounceDelay = 5;
int counter;

float required_weight = 0;
float old_h = 0;
float curr_h = 0;
int servo_current_status = CLOSE;

//LCD Menu Logic
const int numOfScreens = 4;
int currentScreen = 0;
int firstCommand[numOfScreens] = {0, 0, 0, 0};
String screens[numOfScreens][2] = {{"Qunatity", "Grams"}, {"Breakfast", "Hour"}, {"Lunch", "Hour"}, {"Dinner", "Hour"}};
int parameters[numOfScreens][2] = {{ -5, 0}, {0, 0}, {0, 0}, {0, 0}};
Servo servo;

void setup() {
  lcd.init();
  lcd.backlight();

  Serial.begin(9600);
  if (! rtc.begin()) {
    Serial.println("Couldn't find RTC");
    while (1);
  }

  rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  for (int i = 0; i < numOfInputs; i++) {
    pinMode(inputPins[i], INPUT);
    digitalWrite(inputPins[i], HIGH); // pull-up 20k
  }
  Serial.begin(9600);
  lcd.begin(16, 2);
  servo.attach(servoPin);
  servo.write(-30);
  pinMode(ECHOPIN, INPUT);
  pinMode(TRIGPIN, OUTPUT);
  digitalWrite(TRIGPIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGPIN, LOW);
  old_h = microsecondsToCentimeters(pulseIn(ECHOPIN, HIGH));
}

void loop() {
  digitalWrite(TRIGPIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIGPIN, LOW);
  curr_h = microsecondsToCentimeters(pulseIn(ECHOPIN, HIGH));
  now = rtc.now();
  setInputFlags();
  resolveInputFlags();
  counter = 0;
  if (old_h - heightPerDistance * required_weight <= curr_h && servo_current_status == OPEN) {
    old_h = curr_h;
    servo.write(-30);
    servo_current_status = CLOSE;
  }

  for (int i = 1; i <= 3; i++) {
    if (now.hour() == parameters[i][0] && now.minute() == parameters[i][1]) {
      if (counter == 0) {
        servo_current_status = OPEN;
        servo.write(60);
        delay(1000);
      }
      counter++;
    }
  }
}

void setInputFlags() {
  for (int i = 0; i < numOfInputs; i++) {
    int reading = digitalRead(inputPins[i]);
    if (reading != lastInputState[i]) {
      lastDebounceTime[i] = millis();
    }
    if ((millis() - lastDebounceTime[i]) > debounceDelay) {
      if (reading != inputState[i]) {
        inputState[i] = reading;
        if (inputState[i] == HIGH) {
          inputFlags[i] = HIGH;
        }
      }
    }
    lastInputState[i] = reading;
  }
}

void resolveInputFlags() {
  for (int i = 0; i < numOfInputs; i++) {
    if (inputFlags[i] == HIGH) {
      inputAction(i);
      inputFlags[i] = LOW;
      printScreen();
    }
  }
}

void inputAction(int input) {
  if (input == 0) {
    if (currentScreen == 0) {
      currentScreen = numOfScreens - 1;
    } else {
      currentScreen--;
    }
  } else if (input == 1) {
    if (currentScreen == numOfScreens - 1) {
      currentScreen = 0;
    } else {
      currentScreen++;
    }
  } else if (input == 2) {
    if (currentScreen == 0) {
      if (firstCommand[currentScreen] == 0) {
        firstCommand[currentScreen] = 1;
      } else {
        if (parameters[0][0] != 0) {
          parameters[0][0] -= 5;
        }
        required_weight = parameters[0][0] + (float)parameters[0][1] / 10;
      }
    } else {
      parameters[currentScreen][0] =  (parameters[currentScreen][0] + 1) % 24;
    }
  } else if (input == 3) {
    if (currentScreen == 0) {
      if (firstCommand[currentScreen] == 0) {
        firstCommand[currentScreen] = 1;
      } else {
        parameters[0][0] += 5;
        required_weight = parameters[0][0] + (float)parameters[0][1] / 10;
      }
    } else {
      parameters[currentScreen][1] =  (parameters[currentScreen][1] + 1) % 60;
    }
  }
}


void printScreen() {
  lcd.clear();
  lcd.print(screens[currentScreen][0]);
  lcd.setCursor(11, 0);

  if (now.hour() < 10) {
    lcd.print("0");
  }
  lcd.print(now.hour());
  lcd.print(":");
  if (now.minute() < 10) {
    lcd.print("0");
  }
  lcd.print(now.minute());


  lcd.setCursor(0, 1);
  lcd.print(screens[currentScreen][1]);
  if (currentScreen != 0) {
    lcd.setCursor(11, 1);
    if (parameters[currentScreen][0] < 10) {
      lcd.print("0");
    }
    lcd.print(parameters[currentScreen][0]);
    lcd.print(":");
    if (parameters[currentScreen][1] < 10) {
      lcd.print("0");
    }
    lcd.print(parameters[currentScreen][1]);
  } else {
    lcd.setCursor(11, 1);
    lcd.print(parameters[currentScreen][0]);
    lcd.print(",");
    lcd.print(parameters[currentScreen][1]);
  }

}


float microsecondsToCentimeters(long microseconds) {
  return microseconds * 0.034 / 2;
}
